package de.intevation.gnv.action.mapviewer;

import java.io.ByteArrayInputStream;
import java.io.InputStream;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Iterator;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import de.intevation.gnv.action.ArtifactDatabaseActionBase;
import de.intevation.gnv.action.mapviewer.parser.ExternalCallParser;
import de.intevation.gnv.action.mapviewer.parser.XMLExternalCallParser;
import de.intevation.gnv.action.sessionmodel.SessionModel;
import de.intevation.gnv.action.sessionmodel.SessionModelFactory;
import de.intevation.gnv.artifactdatabase.client.ArtifactDatabaseClientFactory;
import de.intevation.gnv.artifactdatabase.objects.ArtifactObject;
import de.intevation.gnv.artifactdatabase.objects.map.MapService;

/**
 * 
 * @author Tim Englich <tim.englich@intevation.de>
 *
 */
public class MapViewerCallAction extends ArtifactDatabaseActionBase {

    /**
     * the logger, used to log exceptions and additonaly information
     */
    private static Logger log = Logger.getLogger(MapViewerCallAction.class);

    /**
     * Constructor
     */
    public MapViewerCallAction() {
        super();
    }

    /**
     * @see de.intevation.gnv.action.ArtifactDatabaseActionBase#execute(org.apache.struts.action.ActionMapping, org.apache.struts.action.ActionForm, javax.servlet.http.HttpServletRequest, javax.servlet.http.HttpServletResponse)
     */
    @Override
    public ActionForward execute(ActionMapping mapping, ActionForm form,
                                 HttpServletRequest request,
                                 HttpServletResponse response) throws Exception {
        log.debug("MapViewerCallAction.execute");
        InputStream inputStream = null;
        String documentvalue = request.getParameter("document");
        if (documentvalue != null){
            inputStream = new ByteArrayInputStream(documentvalue.getBytes());
        }else{
            inputStream = request.getInputStream();
        }
        
        if (inputStream != null){
            try {
                ExternalCallParser ecp = new XMLExternalCallParser(inputStream);
                ecp.parse();
                String geometry = ecp.getGeometry();
                String srs = ecp.getSRS();
                Collection<MapService> mapServices = ecp.getMapServices();
                 
                Collection<ArtifactObject> availableFactories = 
                       ArtifactDatabaseClientFactory
                          .getInstance()
                          .getArtifactDatabaseClient(getLocale(request))
                          .getArtifactFactoryMetaInformation(mapServices,
                                                             geometry,
                                                             srs);
                
                Collection<ArtifactObject> providedFactories = 
                        ArtifactDatabaseClientFactory
                           .getInstance()
                           .getArtifactDatabaseClient(getLocale(request))
                           .getArtifactFactories();
                
                // Sort out which ArtifactFactories should not be used.
                Collection<ArtifactObject> usedFactories = 
                       new ArrayList<ArtifactObject>(availableFactories.size());
                
                Iterator<ArtifactObject> it = availableFactories.iterator();
                while (it.hasNext()){
                    ArtifactObject ao = it.next();
                    if (providedFactories.contains(ao)){
                        usedFactories.add(ao);
                    }
                }
                    
                SessionModel sm = SessionModelFactory
                                     .getInstance()
                                     .getSessionModel(request);
                sm.resetModel();
                sm.setArtifacteFactories(usedFactories);
            } catch (Exception e) {
                log.error(e,e);
            }
        }else{
            log.error("Kein Anfragedokument übergeben.");
        }
        return super.execute(mapping, form, request, response);
    }

}
